
import java.awt.Button;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Stack;

public class UE20Aufgabe24 {
	public static void main(String[] args) {
		RPNCalculator calulator = new RPNCalculator();
		calulator.setVisible(true);
	}
}
interface Operator {
	public int execute(int value1, int value2) throws ArithmeticException;
}

class CalculatorButton extends Button {

	protected RPNCalculator calculator;

	public CalculatorButton(RPNCalculator r) {
		this.calculator = r;
	}
}

class DigitButton extends CalculatorButton implements ActionListener {
	int digit;

	public DigitButton(int digit, RPNCalculator calculator) {
		super(calculator);
		this.digit = digit;
		this.setLabel("" + digit);
		this.addActionListener(this);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.calculator.digitPressed(this.digit);
	}

}

abstract class OperatorButton extends CalculatorButton implements Operator,
		ActionListener {

	public OperatorButton(char sign, RPNCalculator c) {
		super(c);
		this.setLabel("" + sign);
		this.addActionListener(this);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.calculator.operatorPressed(this);
	}

	@Override
	public abstract int execute(int value1, int value2);
}

class AddButton extends OperatorButton {

	public AddButton(RPNCalculator c) {
		super('+', c);
	}

	@Override
	public int execute(int value1, int value2) {
		return value1 + value2;
	}
}

class SubButton extends OperatorButton {

	public SubButton(RPNCalculator c) {
		super('-', c);
	}

	@Override
	public int execute(int value1, int value2) {
		return value1 - value2;
	}
}

class MultButton extends OperatorButton {

	public MultButton(RPNCalculator c) {
		super('*', c);
	}

	@Override
	public int execute(int value1, int value2) {
		return value1 * value2;
	}
}

class DivButton extends OperatorButton {

	public DivButton(RPNCalculator c) {
		super('/', c);
	}

	@Override
	public int execute(int value1, int value2) {
		return value1 / value2;
	}
}

class EnterButton extends CalculatorButton implements ActionListener {
	public EnterButton(RPNCalculator c) {
		super(c);
		this.setLabel("Enter");
		this.addActionListener(this);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.calculator.enterPressed();
	}
}

enum Action {
	OP_PRESSED, ENTER_PRESSED, DIGIT_PRESSED
}

class RPNCalculator extends Frame implements WindowListener {

	Label display;
	Stack<Integer> stack;
	int displayedNumber;
	Action lastAction;

	public RPNCalculator() {
		super("UPN-Rechner");
		this.addWindowListener(this);
		this.stack = new Stack<Integer>();
		this.displayedNumber = 0;
		this.lastAction = Action.ENTER_PRESSED;

		this.setLayout(new GridLayout(6, 1));

		this.display = new Label("" + this.displayedNumber);
		this.add(this.display);

		Panel p = new Panel();
		p.setLayout(new GridLayout(1, 4));
		p.add(new DigitButton(7, this));
		p.add(new DigitButton(8, this));
		p.add(new DigitButton(9, this));
		p.add(new AddButton(this));
		this.add(p);

		p = new Panel();
		p.setLayout(new GridLayout(1, 4));
		p.add(new DigitButton(4, this));
		p.add(new DigitButton(5, this));
		p.add(new DigitButton(6, this));
		p.add(new SubButton(this));
		this.add(p);

		p = new Panel();
		p.setLayout(new GridLayout(1, 4));
		p.add(new DigitButton(1, this));
		p.add(new DigitButton(2, this));
		p.add(new DigitButton(3, this));
		p.add(new MultButton(this));
		this.add(p);

		p = new Panel();
		p.setLayout(new GridLayout(1, 4));
		p.add(new Button(""));
		p.add(new DigitButton(0, this));
		p.add(new Button(""));
		p.add(new DivButton(this));
		this.add(p);

		this.add(new EnterButton(this));

		this.pack();
	}

	public void digitPressed(int digit) {
		if (this.lastAction == Action.ENTER_PRESSED) {
			this.displayedNumber = digit;
		} else if (this.lastAction == Action.DIGIT_PRESSED) {
			this.displayedNumber = this.displayedNumber * 10 + digit;
		} else if (this.lastAction == Action.OP_PRESSED) {
			this.stack.push(this.displayedNumber);
			this.displayedNumber = digit;
		}
		this.lastAction = Action.DIGIT_PRESSED;
		this.display.setText("" + this.displayedNumber);
	}

	public void operatorPressed(Operator op) {
		int value = 0;
		if (!this.stack.isEmpty()) {
			value = this.stack.pop();
		}

		try {
			this.displayedNumber = op.execute(value, this.displayedNumber);
			this.display.setText("" + this.displayedNumber);
		} catch (ArithmeticException exc) {
			this.stack.clear();
			this.display.setText("ERROR");
		}
		this.lastAction = Action.OP_PRESSED;
	}

	public void enterPressed() {
		this.stack.push(this.displayedNumber);
		this.lastAction = Action.ENTER_PRESSED;
	}



	@Override
	public void windowOpened(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowClosing(WindowEvent e) {
		System.exit(0);
		
	}

	@Override
	public void windowClosed(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowIconified(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowDeiconified(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowActivated(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void windowDeactivated(WindowEvent e) {
		// TODO Auto-generated method stub
		
	}

}

