import java.util.ArrayList;

interface DoubleSideStackI {

	// liefert genau dann true, wenn sich kein Element auf dem DoubleSideStack
	// befindet
	public boolean isEmpty();

	// fuegt den uebergebenen Wert auf der rechten Seite des DoubleSideStacks an
	public void pushRight(int value);

	// entfernt das am weitesten rechts liegende Element aus dem DoubleSideStack
	// und liefert es als Methodenwert; ist der DoubleSideStack leer, wird eine
	// java.lang.IndexOutOfBoundsException als Uncheck-Exception geworfen
	public int popRight();

	// fuegt den uebergebenen Wert auf der linken Seite des DoubleSideStacks an
	public void pushLeft(int value);

	// entfernt das am weitesten links liegende Element aus dem DoubleSideStack
	// und liefert es als Methodenwert; ist der DoubleSideStack leer, wird eine
	// java.lang.IndexOutOfBoundsException als Uncheck-Exception geworfen
	public int popLeft();

	// liefert einen String, in dem die Werte der Elemente des DoubleSideStack
	// von links nach rechts enthalten sind, nach jedem Wert steht ein
	// Leerzeichen; Beispiel: "12 2 -34 "
	public String toString();

	// liefert einen neuen DoubleSideStack, der dieselben Werte in derselben
	// Reihenfolge wie der aufgerufene DoubleSideStack speichert; Achtung: Die
	// beiden DoubleSideStacks sollen unabhaengig voneinander sein, d.h. sie
	// sollen sich keine Elemente teilen!
	public DoubleSideStackI clone();

	// liefert genau dann true; wenn beide DoubleSideStacks dieselben Werte in
	// derselben Reihenfolge speichern
	public boolean equals(Object obj);
}

class DoubleSideStack implements DoubleSideStackI {

	private ArrayList<Integer> elems;

	public DoubleSideStack() {
		elems = new ArrayList<Integer>();
	}

	public DoubleSideStack(DoubleSideStack stack) {
		this();
		for (int value : stack.elems) {
			elems.add(value);
		}
	}

	@Override
	public DoubleSideStackI clone() {
		return new DoubleSideStack(this);
	}

	public boolean isEmpty() {
		return elems.isEmpty();
	}

	public void pushRight(int value) {
		elems.add(value);
	}

	public int popRight() {
		return elems.remove(elems.size() - 1);
	}

	public void pushLeft(int value) {
		elems.add(0, value);
	}

	public int popLeft() {
		return elems.remove(0);
	}

	@Override
	public String toString() {
		String result = "";
		for (int i = 0; i < elems.size(); i++) {
			result += elems.get(i) + " ";
		}
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		DoubleSideStack other = (DoubleSideStack) obj;
		if (elems == null) {
			if (other.elems != null) {
				return false;
			}
		} else if (!elems.equals(other.elems)) {
			return false;
		}
		return true;
	}

	@Override
	// nicht gefordert
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((elems == null) ? 0 : elems.hashCode());
		return result;
	}

}

public class UE20Aufgabe32 {

	// Testprogramm (nicht gefordert)
	public static void main(String[] args) {
		DoubleSideStack stack = new DoubleSideStack();
		stack.pushLeft(23);
		stack.pushLeft(-34);
		stack.pushRight(5);
		stack.pushLeft(-8);
		stack.pushRight(67);
		System.out.println(stack.toString());
		stack.popLeft();
		stack.popRight();
		System.out.println(stack.toString());
	}

}
