import java.awt.Button;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JFrame;

class GameManager extends Observable {

	// Konstanten sollen prinzipiell geaendert werden koennen, ohne den Code
	// aendern zu muessen!
	final public static int ANZAHL_MUENZEN = 12; // groesser als 1
	final public static int ANZAHL_SPIELER = 3; // groesser als 1
	final public static int MIN_MUENZWERT = 1; // groesser als 0
	final public static int MAX_MUENZWERT = 6; // groesser als MIN_MUENZWERT

	private int[] muenzWerte;
	private int[] punkte;
	private int aktuellerSpielerIndex;

	public GameManager() {
		this.muenzWerte = new int[GameManager.ANZAHL_MUENZEN];
		for (int i = 0; i < GameManager.ANZAHL_MUENZEN; i++) {
			this.muenzWerte[i] = GameManager.getRandomNumber(
					GameManager.MIN_MUENZWERT, GameManager.MAX_MUENZWERT);
		}
		this.punkte = new int[GameManager.ANZAHL_SPIELER];
		this.aktuellerSpielerIndex = 0;
	}

	// liefert Zufallszahl zwischen min und max
	private static int getRandomNumber(int min, int max) {
		return (int) (min + (Math.random() * (max - min) + 1));
	}

	public int getMuenzWert(int index) {
		return this.muenzWerte[index];
	}

	public int getPunkte(int index) {
		return this.punkte[index];
	}

	public int getAktuellerSpielerIndex() {
		return this.aktuellerSpielerIndex;
	}

	public int getAktuellerIndexLinks() {
		for (int i = 0; i < this.muenzWerte.length; i++) {
			if (this.muenzWerte[i] != 0) {
				return i;
			}
		}
		return -1;
	}

	public int getAktuellerIndexRechts() {
		for (int i = this.muenzWerte.length - 1; i >= 0; i--) {
			if (this.muenzWerte[i] != 0) {
				return i;
			}
		}
		return -1;
	}

	public void spielzug(int index) {
		this.punkte[this.aktuellerSpielerIndex] += this.muenzWerte[index];
		this.muenzWerte[index] = 0;
		this.aktuellerSpielerIndex = (this.aktuellerSpielerIndex + 1)
				% GameManager.ANZAHL_SPIELER;
		this.setChanged();
		this.notifyObservers();
	}

	public boolean spielEnde() {
		for (int i = 0; i < this.muenzWerte.length; i++) {
			if (this.muenzWerte[i] != 0) {
				return false;
			}
		}
		return true;
	}

	public boolean istSieger(int index) {
		if (!this.spielEnde()) {
			return false;
		}
		int max = 0;
		for (int i = 0; i < this.punkte.length; i++) {
			if (this.punkte[i] > max) {
				max = this.punkte[i];
			}
		}
		return this.punkte[index] == max;
	}
}

class SpielerLabel extends Label implements Observer {
	GameManager manager;
	int myIndex;

	public SpielerLabel(GameManager m, int index) {
		super("Spieler " + (index + 1) + " hat " + m.getPunkte(index)
				+ " Punkte");
		this.manager = m;
		this.myIndex = index;
		m.addObserver(this);
	}

	@Override
	public void update(Observable o, Object arg) {
		String msg = "Spieler " + (this.myIndex + 1) + " hat "
				+ this.manager.getPunkte(this.myIndex) + " Punkte!";
		if (this.manager.istSieger(this.myIndex)) {
			msg += " Sieger!";
		}
		this.setText(msg);
	}
}

class MessageLabel extends Label implements Observer {
	GameManager manager;

	public MessageLabel(GameManager m) {
		super("Spieler " + (m.getAktuellerSpielerIndex() + 1) + " ist am Zug!");
		this.manager = m;
		m.addObserver(this);
	}

	@Override
	public void update(Observable o, Object arg) {
		if (!this.manager.spielEnde()) {
			this.setText("Spieler "
					+ (this.manager.getAktuellerSpielerIndex() + 1)
					+ " ist am Zug!");
		} else {
			this.setText("Spielende!");
		}
	}
}

class MuenzButton extends Button implements ActionListener, Observer {
	GameManager manager;
	int myIndex;

	public MuenzButton(GameManager m, int index) {
		super("" + m.getMuenzWert(index));
		this.manager = m;
		this.myIndex = index;
		this.addActionListener(this);
		m.addObserver(this);
		this.checkEnabled();
		this.setFocusable(false);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.manager.spielzug(this.myIndex);
	}

	@Override
	public void update(Observable o, Object arg) {
		this.checkEnabled();
	}

	private void checkEnabled() {
		int wert = this.manager.getMuenzWert(this.myIndex);
		if (wert > 0) {
			this.setLabel("" + wert);
		} else {
			this.setLabel("");
		}
		if (this.myIndex == this.manager.getAktuellerIndexLinks()
				|| this.myIndex == this.manager.getAktuellerIndexRechts()) {
			this.setBackground(Color.GREEN);
			this.setEnabled(true);
		} else {
			this.setBackground(Color.WHITE);
			this.setEnabled(false);
		}
	}
}

public class UE20Aufgabe37 {

	public static void main(String[] args) {
		new UE20Aufgabe37(new GameManager());
	}

	private JFrame fenster;
	private MuenzButton[] muenzButtons;
	private MessageLabel messageLabel;
	private SpielerLabel[] spielerLabel;

	public UE20Aufgabe37(GameManager manager) {
		this.fenster = new JFrame("Münzen-Spiel");
		this.fenster.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.fenster.setTitle("Münzen-Spiel");
		this.fenster.setLayout(new GridLayout(2, 0));

		Panel buttonPanel = new Panel();
		buttonPanel.setLayout(new GridLayout(1, 0));
		this.muenzButtons = new MuenzButton[GameManager.ANZAHL_MUENZEN];
		for (int i = 0; i < GameManager.ANZAHL_MUENZEN; i++) {
			this.muenzButtons[i] = new MuenzButton(manager, i);
			buttonPanel.add(this.muenzButtons[i]);
		}
		this.fenster.add(buttonPanel);

		Panel messagesPanel = new Panel();
		messagesPanel.setLayout(new GridLayout(1, 0));
		this.messageLabel = new MessageLabel(manager);
		messagesPanel.add(this.messageLabel);

		Panel spielerPanel = new Panel();
		spielerPanel.setLayout(new GridLayout(0, 1));
		this.spielerLabel = new SpielerLabel[GameManager.ANZAHL_SPIELER];
		for (int i = 0; i < GameManager.ANZAHL_SPIELER; i++) {
			this.spielerLabel[i] = new SpielerLabel(manager, i);
			spielerPanel.add(this.spielerLabel[i]);
		}
		messagesPanel.add(spielerPanel);
		this.fenster.add(messagesPanel);

		this.fenster.pack();
		this.fenster.setLocation(50, 50);
		this.fenster.setVisible(true);
	}

}
