import java.awt.Button;
import java.awt.Color;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/*
 * Punkte: Gesamt = 40
 * 
 * - GUI: 15
 * - Spielerwechsel: 3
 * - Setzen der Buttons (Farbe/Beschriftung/Disabled) bei Anklicken: 4
 * - Setzen der Label (Punkte): 3
 * - Überprüfen auf Vierer-Blöcke: 10
 * - Spielende: 5
 * 
 */
public class UE20Aufgabe44 {

	public static void main(String[] args) {
		Board board = new Board();
		board.build();
	}

}

class Board {
	final static int SIZE = 6;
	final static int PLAYER_1 = 0;
	final static int PLAYER_2 = 1;
	final static int EMPTY = -1;
	int[][] board;
	int[] score;
	Button[][] buttons;
	Label[] scoreLabels;

	public Board() {
		this.board = new int[SIZE][SIZE];
		this.buttons = new Button[SIZE][SIZE];
		for (int r = 0; r < SIZE; r++) {
			for (int c = 0; c < SIZE; c++) {
				this.board[r][c] = EMPTY;
				this.buttons[r][c] = new Button(" ");
				this.buttons[r][c].setBackground(Color.GRAY);
				this.buttons[r][c]
						.addActionListener(new ClickListener(this, r, c));
			}
		}
		this.score = new int[2];
		this.scoreLabels = new Label[2];
		for (int i = 0; i < 2; i++) {
			this.score[i] = 0;
			this.scoreLabels[i] = new Label(
					"Spieler " + (i + 1) + ": " + this.score[i]);
		}
	}

	public void build() {
		Frame frame = new Frame("Four Blocks");
		GridLayout frameLayout = new GridLayout(2, 0);
		frame.setLayout(frameLayout);
		Panel labelPanel = new Panel();
		GridLayout labelLayout = new GridLayout(0, 2);
		labelPanel.setLayout(labelLayout);
		for (int i = 0; i < 2; i++) {
			labelPanel.add(this.scoreLabels[i]);
		}
		Panel buttonPanel = new Panel();
		GridLayout buttonLayout = new GridLayout(SIZE, SIZE);
		buttonPanel.setLayout(buttonLayout);
		for (int r = 0; r < SIZE; r++) {
			for (int c = 0; c < SIZE; c++) {
				buttonPanel.add(this.buttons[r][c]);
			}
		}
		frame.add(labelPanel);
		frame.add(buttonPanel);

		frame.setBounds(10, 10, 600, 800);
		frame.setVisible(true);
	}

	public int[][] getBoard() {
		return board;
	}

	public int[] getScore() {
		return score;
	}

	public Button[][] getButtons() {
		return buttons;
	}

	public Label[] getScoreLabels() {
		return scoreLabels;
	}
}

class ClickListener implements ActionListener {

	Board board;
	int row;
	int col;
	static int currentPlayer = 0;

	public ClickListener(Board board, int row, int col) {
		this.board = board;
		this.row = row;
		this.col = col;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.board.getBoard()[this.row][this.col] = currentPlayer;
		this.board.getButtons()[this.row][this.col]
				.setLabel("" + (currentPlayer + 1));
		if (currentPlayer == 0) {
			this.board.getButtons()[this.row][this.col]
					.setBackground(Color.GREEN);
		} else {
			this.board.getButtons()[this.row][this.col]
					.setBackground(Color.YELLOW);
		}

		this.board.getButtons()[this.row][this.col].setEnabled(false);
		checkFourBlocks();
		checkEnd();
		currentPlayer = (currentPlayer + 1) % 2;
	}

	void checkFourBlocks() {
		int[][] fields = this.board.getBoard();

		// links oben
		if (row > 0 && col > 0 && fields[row - 1][col - 1] == currentPlayer
				&& fields[row - 1][col] == currentPlayer
				&& fields[row][col - 1] == currentPlayer) {
			this.board.getScore()[currentPlayer]++;
		}
		// rechts oben
		if (row > 0 && col < Board.SIZE - 1
				&& fields[row - 1][col] == currentPlayer
				&& fields[row - 1][col + 1] == currentPlayer
				&& fields[row][col + 1] == currentPlayer) {
			this.board.getScore()[currentPlayer]++;
		}
		// links unten
		if (row < Board.SIZE - 1 && col > 0
				&& fields[row][col - 1] == currentPlayer
				&& fields[row + 1][col - 1] == currentPlayer
				&& fields[row + 1][col] == currentPlayer) {
			this.board.getScore()[currentPlayer]++;
		}
		// rechts unten
		if (row < Board.SIZE - 1 && col < Board.SIZE - 1
				&& fields[row][col + 1] == currentPlayer
				&& fields[row + 1][col] == currentPlayer
				&& fields[row + 1][col + 1] == currentPlayer) {
			this.board.getScore()[currentPlayer]++;
		}
		this.board.getScoreLabels()[currentPlayer]
				.setText("Spieler " + (currentPlayer + 1) + ": "
						+ this.board.getScore()[currentPlayer]);
	}

	void checkEnd() {
		for (int r = 0; r < Board.SIZE; r++) {
			for (int c = 0; c < Board.SIZE; c++) {
				if (this.board.getBoard()[r][c] == Board.EMPTY) {
					return;
				}
			}
		}
		if (this.board.getScore()[0] > this.board.getScore()[1]) {
			this.board.getScoreLabels()[0].setText("Spieler 1: Gewonnen!");
			this.board.getScoreLabels()[1].setText("Spieler 2: Verloren!");
		} else if (this.board.getScore()[0] < this.board.getScore()[1]) {
			this.board.getScoreLabels()[0].setText("Spieler 1: Verloren!");
			this.board.getScoreLabels()[1].setText("Spieler 2: Gewonnen!");
		} else {
			this.board.getScoreLabels()[0].setText("Spieler 1: Unentschieden!");
			this.board.getScoreLabels()[1].setText("Spieler 2: Unentschieden!");
		}
	}

}
