import java.awt.Button;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;

public class UE20Aufgabe45 {

	public static void main(String[] args) {
		Frame2048 frame = new Frame2048();
		frame.setLocation(600, 300);
		frame.setVisible(true);
	}
}

class Frame2048 extends JFrame {

	private Label[][] fields;
	private Button up, down, left, right;
	private Label messageLabel;
	private Board board;

	public Frame2048() {
		super("Spiel 2048");
		this.board = new Board();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLayout(new GridLayout(3, 1));
		Panel gamePanel = new Panel();
		gamePanel.setLayout(new GridLayout(Board.SIZE, Board.SIZE));
		fields = new Label[Board.SIZE][Board.SIZE];
		for (int r = 0; r < Board.SIZE; r++) {
			for (int c = 0; c < Board.SIZE; c++) {
				fields[r][c] = new Label("" + board.getNumber(r, c));
				fields[r][c].setAlignment(Label.CENTER);
				gamePanel.add(fields[r][c]);
			}
		}
		this.add(gamePanel);

		Panel controlPanel = new Panel();
		controlPanel.setLayout(new GridLayout(1, 4));
		up = new Button("^");
		up.addActionListener(new UpListener(this));
		controlPanel.add(up);
		down = new Button("v");
		down.addActionListener(new DownListener(this));
		controlPanel.add(down);
		left = new Button("<");
		left.addActionListener(new LeftListener(this));
		controlPanel.add(left);
		right = new Button(">");
		right.addActionListener(new RightListener(this));
		controlPanel.add(right);
		this.add(controlPanel);

		messageLabel = new Label("Willkommen beim Spiel 2048");
		this.add(messageLabel);

		this.setSize(600, 400);
	}

	public void update() {
		for (int r = 0; r < Board.SIZE; r++) {
			for (int c = 0; c < Board.SIZE; c++) {
				fields[r][c].setText("" + board.getNumber(r, c));
			}
		}
	}

	public void won() {
		up.setEnabled(false);
		down.setEnabled(false);
		left.setEnabled(false);
		right.setEnabled(false);
		messageLabel.setText("Gewonnen! Herzlichen Glückwunsch!");
	}
	
	public void lost() {
		up.setEnabled(false);
		down.setEnabled(false);
		left.setEnabled(false);
		right.setEnabled(false);
		messageLabel.setText("Leider verloren!");
	}

	public Board getBoard() {
		return board;
	}
}

class Board {
	public static int SIZE = 4;

	private int[][] field;

	public Board() {
		field = new int[SIZE][SIZE];
		for (int r = 0; r < SIZE; r++) {
			for (int c = 0; c < SIZE; c++) {
				field[r][c] = 0;
			}
		}
		addRandom();
		addRandom();
	}

	public int getNumber(int row, int col) {
		return field[row][col];
	}

	public void setNumber(int row, int col, int newNumber) {
		field[row][col] = newNumber;
	}

	public boolean isEmpty(int row, int col) {
		return field[row][col] == 0;
	}

	public void clear(int row, int col) {
		field[row][col] = 0;
	}

	public boolean duplicate(int row, int col) {
		field[row][col] *= 2;
		return field[row][col] == 2048;
	}

	public void addRandom() {
		int r;
		int c;
		do {
			r = (int) (Math.random() * SIZE);
			c = (int) (Math.random() * SIZE);
		} while (field[r][c] != 0);
		field[r][c] = 2;
	}

	public boolean isFull() {
		for (int r = 0; r < SIZE; r++) {
			for (int c = 0; c < SIZE; c++) {
				if (field[r][c] == 0) {
					return false;
				}
			}
		}
		return true;
	}
}

class UpListener implements ActionListener {

	private Frame2048 frame;
	private boolean won;

	public UpListener(Frame2048 frame) {
		this.frame = frame;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		boolean action = false;
		won = false;
		for (int r = 1; r < Board.SIZE; r++) {
			for (int c = 0; c < Board.SIZE; c++) {
				action = up(r, c) || action;
			}
		}
		if (won) {
			frame.update();
			frame.won();
			return;
		}
		if (frame.getBoard().isFull()) {
			frame.update();
			frame.lost();
			return;
		}
		if (action) {
			frame.getBoard().addRandom();
			frame.update();
		}
	}

	public boolean up(int row, int col) {
		Board board = frame.getBoard();
		int number = board.getNumber(row, col);
		if (number == 0) {
			return false;
		}
		for (int r = row - 1; r >= 0; r--) {
			if (number == board.getNumber(r, col)) {
				won = board.duplicate(r, col) || won;
				board.clear(row, col);
				return true;
			}
			if (!board.isEmpty(r, col)) {
				if (r != row - 1) {
					board.setNumber(r + 1, col, number);
					board.clear(row, col);
					return true;
				} else {
					return false;
				}
			}
		}
		board.setNumber(0, col, number);
		board.clear(row, col);
		return true;
	}
}

class LeftListener implements ActionListener {

	private Frame2048 frame;
	private boolean won;

	public LeftListener(Frame2048 frame) {
		this.frame = frame;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		boolean action = false;
		won = false;
		for (int r = 0; r < Board.SIZE; r++) {
			for (int c = 1; c < Board.SIZE; c++) {
				action = left(r, c) || action;
			}
		}
		if (won) {
			frame.won();
			return;
		}
		if (frame.getBoard().isFull()) {
			frame.update();
			frame.lost();
			return;
		}
		if (action) {
			frame.getBoard().addRandom();
			frame.update();
		}
	}

	public boolean left(int row, int col) {
		Board board = frame.getBoard();
		int number = board.getNumber(row, col);
		if (number == 0) {
			return false;
		}
		for (int c = col - 1; c >= 0; c--) {
			if (number == board.getNumber(row, c)) {
				won = board.duplicate(row, c) || won;
				board.clear(row, col);
				return true;
			}
			if (!board.isEmpty(row, c)) {
				if (c != col - 1) {
					board.setNumber(row, c + 1, number);
					board.clear(row, col);
					return true;
				} else {
					return false;
				}
			}
		}
		board.setNumber(row, 0, number);
		board.clear(row, col);
		return true;
	}
}

class DownListener implements ActionListener {

	private Frame2048 frame;
	private boolean won;

	public DownListener(Frame2048 frame) {
		this.frame = frame;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		boolean action = false;
		won = false;
		for (int r = Board.SIZE - 2; r >= 0; r--) {
			for (int c = Board.SIZE - 1; c >= 0; c--) {
				action = down(r, c) || action;
			}
		}
		if (won) {
			frame.update();
			frame.won();
			return;
		}
		if (frame.getBoard().isFull()) {
			frame.update();
			frame.lost();
			return;
		}
		if (action) {
			frame.getBoard().addRandom();
			frame.update();
		}
	}

	public boolean down(int row, int col) {
		Board board = frame.getBoard();
		int number = board.getNumber(row, col);
		if (number == 0) {
			return false;
		}
		for (int r = row + 1; r < Board.SIZE; r++) {
			if (number == board.getNumber(r, col)) {
				won = board.duplicate(r, col) || won;
				board.clear(row, col);
				return true;
			}
			if (!board.isEmpty(r, col)) {
				if (r != row + 1) {
					board.setNumber(r - 1, col, number);
					board.clear(row, col);
					return true;
				} else {
					return false;
				}
			}
		}
		board.setNumber(Board.SIZE - 1, col, number);
		board.clear(row, col);
		return true;
	}
}

class RightListener implements ActionListener {

	private Frame2048 frame;
	private boolean won;

	public RightListener(Frame2048 frame) {
		this.frame = frame;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		boolean action = false;
		won = false;
		for (int r = Board.SIZE - 1; r >= 0; r--) {
			for (int c = Board.SIZE - 2; c >= 0; c--) {
				action = right(r, c) || action;
			}
		}
		if (won) {
			frame.won();
			return;
		}
		if (frame.getBoard().isFull()) {
			frame.update();
			frame.lost();
			return;
		}
		if (action) {
			frame.getBoard().addRandom();
			frame.update();
		}
	}

	public boolean right(int row, int col) {
		Board board = frame.getBoard();
		int number = board.getNumber(row, col);
		if (number == 0) {
			return false;
		}
		for (int c = col + 1; c < Board.SIZE; c++) {
			if (number == board.getNumber(row, c)) {
				won = board.duplicate(row, c) || won;
				board.clear(row, col);
				return true;
			}
			if (!board.isEmpty(row, c)) {
				if (c != col + 1) {
					board.setNumber(row, c - 1, number);
					board.clear(row, col);
					return true;
				} else {
					return false;
				}
			}
		}
		board.setNumber(row, Board.SIZE - 1, number);
		board.clear(row, col);
		return true;
	}
}
